################################################################################
#   Copyright 2011 Chris Statzer and David Reichard                            #
#                                                                              #
#   Licensed under the Apache License, Version 2.0 (the "License");            #
#   you may not use this file except in compliance with the License            #
#   You may obtain a copy of the License at                                    #
#                                                                              #
#       http://www.apache.org/licenses/LICENSE-2.0                             #
#                                                                              #
#   Unless required by applicable law or agreed to in writing, software        #
#   distributed under the License is distributed on an "AS IS" BASIS,          #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   #
#   See the License for the specific language governing permissions and        #
#   limitations under the License.                                             #
################################################################################

import pyglet
from random import randint


###############################################################################
################################################################ Bubble ####### 
###############################################################################

class Bubble (pyglet.sprite.Sprite):

    def __init__(self, image, x, y, size, speed):
        pyglet.sprite.Sprite.__init__(self, image, x=x, y=y)
        self.scale = size
        self.speed = speed


###############################################################################
############################################################### BubbleEmitter # 
###############################################################################

class BubbleEmitter (object):

    def __init__(self, window):
        self.window = window
        self._bubbles = []
        self.bubble_images = self._load_bubbles()

    def update(self, dt):
        #print "updating emitter", dt
        if len(self._bubbles) <= 20:
            self.add_bubble()
                        
        for b in self._bubbles:
            if b.y >= self.window.height:
                self._bubbles.remove(b)
            else:
                b.y += b.speed * dt
        
    def draw(self):
        for b in self._bubbles:
            b.draw()
    
    def add_bubble(self):
        size = randint(0,3)-1
        self._bubbles.append(Bubble(self.bubble_images[size], randint(0, self.window.width), 0, size,randint(50,200)))

    def _load_bubbles(self):
        bubbs = []
        for n in range(4):
            image = pyglet.image.load('assets/bubble.png')
            bubbs.append(image)
        return bubbs


class PositionalBubbleEmitter(BubbleEmitter):
    def __init__(self, window, sprite):
        BubbleEmitter.__init__(self, window)
        self.sprite = sprite

    def update(self, dt):
        
        for b in self._bubbles:
            if b.y >= 1450: ## Magic Number alert
                self._bubbles.remove(b)
            else:
                b.y += b.speed * dt

        if len(self._bubbles) >= 20:
            return

        if self.sprite.velocity.x != 0:
            if randint(1,100) <= 5:
                self.add_bubble()

    def add_bubble(self):
            size = randint(0,3)-1
            self._bubbles.append(Bubble(self.bubble_images[size], self.sprite.x, self.sprite.y, size,randint(50,200)))


