################################################################################
#   Copyright 2011 Chris Statzer and David Reichard                            #
#                                                                              #
#   Licensed under the Apache License, Version 2.0 (the "License");            #
#   you may not use this file except in compliance with the License            #
#   You may obtain a copy of the License at                                    #
#                                                                              #
#       http://www.apache.org/licenses/LICENSE-2.0                             #
#                                                                              #
#   Unless required by applicable law or agreed to in writing, software        #
#   distributed under the License is distributed on an "AS IS" BASIS,          #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   #
#   See the License for the specific language governing permissions and        #
#   limitations under the License.                                             #
################################################################################

import math
from vec2d import vec2d

def line_intersect(p1, p2, p3, p4):
    S1x = p2.x - p1.x
    S1y = p2.y - p1.y
    
    S2x = p4.x - p3.x
    S2y = p4.y - p3.y
    
    denominator = (-S2x * S1y + S1x * S2y)
    
    # lines are either parallel or coincident if denominator is 0
    if denominator == 0:
        return None
    
    s = (-S1y * (p1.x - p3.x) + S1x * (p1.y - p3.y)) / denominator
    t = (S2x * (p1.y - p3.y) - S2y * (p1.x - p3.x)) / denominator
    
    if not (0 <= s and s <= 1 and 0 <= t and t <= 1):
        return None
    
    Ux = p1.x + t * S1x
    Uy = p1.y + t * S1y
    
    return vec2d(Ux, Uy)

def calc_new_pos(playerStart, playerEnd, lineStart, lineEnd, eps = 0.4):
    line = lineEnd - lineStart
    
    normal = line.perpendicular_normal()
    
    d = normal.dot(lineStart)
    
    startDist = normal.dot(playerStart) - d
    
    if startDist < 0.0:
        normal *= -1
        d *= -1
        
    endDist = normal.dot(playerEnd) - d
    

    return playerEnd + normal * (-endDist + eps)

def circle_point_intersect(circle_pos, radius, point):
    return (point - circle_pos).length < radius

def circle_circle_intersect(cp1, rad1, cp2, rad2):
    return (cp2 - cp1).length < rad1 + rad2
