################################################################################
#   Copyright 2011 Chris Statzer and David Reichard                            #
#                                                                              #
#   Licensed under the Apache License, Version 2.0 (the "License");            #
#   you may not use this file except in compliance with the License            #
#   You may obtain a copy of the License at                                    #
#                                                                              #
#       http://www.apache.org/licenses/LICENSE-2.0                             #
#                                                                              #
#   Unless required by applicable law or agreed to in writing, software        #
#   distributed under the License is distributed on an "AS IS" BASIS,          #
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.   #
#   See the License for the specific language governing permissions and        #
#   limitations under the License.                                             #
################################################################################

import json

import pyglet
from pyglet.gl import *

###############################################################################
################################################################ Terrain ###### 
###############################################################################

class Terrain (object):
    """
        Loads filename.png/col for the image/collision points.
        stores collision points in .walls. A draw with the kwarg
        debug=True will draw the GL_LINES of the boundry
    """
    def __init__(self, filename, x, y):
        self.x, self.y = x, y
        col_file = open('assets/%s.col' % filename)
        points = json.loads(col_file.read())
        self.walls = []
        for xp in points[::2]:
            self.walls.append(xp + x)
        n = 1
        for yp in points[1::2]:
            self.walls.insert(n, yp + y)
            n +=2
        #print points	
        #print self.walls
        self.image = pyglet.resource.image('%s.png' % filename)

    def draw(self, debug=False):
        self.image.blit(self.x, self.y)
        #glColor4f(0.1, 0.9, 0.1, 1.0)
        if debug:
            pyglet.graphics.draw(len(self.walls)/2, GL_LINES,
                ('v2i', self.walls)
            )
