import pyglet
from pyglet.gl import *
from pyglet.window import key
from vec2d import vec2d
from collision import line_intersect, calc_new_pos

window = pyglet.window.Window(width = 1024, height = 768)
fps_display = pyglet.clock.ClockDisplay()
pyglet.resource.path = ['assets']
pyglet.resource.reindex()

cell_image = pyglet.resource.image('cell.png')
cell_image.anchor_x = cell_image.width // 2
cell_image.anchor_y = cell_image.height // 2
cell = pyglet.sprite.Sprite(cell_image, 250, window.height//2)
cell.velocity = vec2d(0.0, 0.0)

lines = [50, 100, 400, 150, 
        400, 150, 400, 300, 
        400, 300, 200, 500,
        200, 500, 200, 599,
        200, 599, 50, 599,
        50, 599, 50, 100]
        
@window.event
def on_key_press(k, mod):
    if k == key.DOWN:
        cell.velocity.y += -100
    elif k == key.UP:
        cell.velocity.y += 100
    elif k == key.LEFT:
        cell.velocity.x += -100
    elif k == key.RIGHT:
        cell.velocity.x += 100
    
@window.event
def on_key_release(k, mod):
    if k == key.DOWN:
        cell.velocity.y -= -100
    elif k == key.UP:
        cell.velocity.y -= 100
    elif k == key.LEFT:
        cell.velocity.x -= -100
    elif k == key.RIGHT:
        cell.velocity.x -= 100
        
@window.event
def on_draw():
    window.clear()
    cell.draw()
    glColor4f(0.1, 0.9, 0.1, 1.0)
    pyglet.graphics.draw(len(lines)/2, GL_LINES,
        ('v2i', lines)
    )
    i = 0
    radius = cell_image.width / 2
    while i < len(lines):
        p1 = vec2d(lines[i], lines[i+1])
        p2 = vec2d(lines[i+2], lines[i+3])
        line = p2 - p1
        normal = line.perpendicular_normal() * -1
        vec = (cell.x, cell.y) + radius * normal
        pyglet.graphics.draw(2, GL_LINES,
            ('v2f', [cell.x, cell.y,
                    vec.x, vec.y])
        )
        i+=4
    fps_display.draw()
    
def on_update(dt):
    # going to store the old position, for collision purposes
    old_pos = vec2d(cell.x, cell.y)
    
    # move the cell towards it's velocity
    cell.x += cell.velocity[0] * dt
    cell.y += cell.velocity[1] * dt
    
    # check for collisions against wall
    # if one is found, get new position, to allow "sliding"
    # against them
    radius = cell_image.width / 2
    i = 0
    if cell.velocity != (0, 0):
        while i < len(lines):
            p1 = vec2d(lines[i], lines[i+1])
            p2 = vec2d(lines[i+2], lines[i+3])
            line = p2 - p1
            normal = line.perpendicular_normal() * -1
            vec = (cell.x, cell.y) + radius * normal
            intersects = line_intersect(old_pos, vec, p1, p2)
            if not intersects is None:
                new_pos = calc_new_pos(old_pos, vec, p1, p2)
                new_pos -= radius * normal
                cell.x = new_pos.x
                cell.y = new_pos.y
            i += 4


glEnable(GL_BLEND)
glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
pyglet.clock.schedule_interval(on_update, 1/120.0)
pyglet.app.run()